=== modified file '.bzrignore'
--- .bzrignore	2005-09-13 23:08:19 +0000
+++ .bzrignore	2005-10-03 16:53:39 +0000
@@ -19,3 +19,4 @@
 ./ID
 ./tags
 .bzr-shelf
+bzrlib/plugins/*

=== modified file 'HACKING'
--- HACKING	2005-10-06 11:26:51 +0000
+++ HACKING	2005-10-06 22:15:52 +0000
@@ -155,6 +155,7 @@
 The policy about how output is presented in the text-mode client
 should be only in the command-line tool.
 
+
 Writing tests
 =============
 In general tests should be placed in a file named testFOO.py where 
@@ -164,11 +165,20 @@
 For example, tests for merge3 in bzrlib belong in bzrlib/tests/testmerge3.py.
 See bzrlib/selftest/testsampler.py for a template test script.
 
+
 Running tests
 =============
 Currently, bzr selftest is used to invoke tests.
 You can provide a pattern argument to run a subset. For example, 
-to run just the whitebox tests, run bzr selftest --pattern .*whitebox.*
+to run just the whitebox tests, run::
+
+  bzr selftest -v whitebox
+
+
+Errors and exceptions
+=====================
+
+[write me]
 
 
 Jargon

=== modified file 'NEWS'
--- NEWS	2005-10-06 07:03:33 +0000
+++ NEWS	2005-10-06 13:08:14 +0000
@@ -1,4 +1,4 @@
-DEVELOPMENT HEAD
+bzr 0.1 2005-10-06
 
 
   BUG FIXES:
@@ -36,6 +36,11 @@
     * 'bzr status' now shows unknown files inside a selected directory.
       Patch from Heikki Paajanen.
 
+    * Merge conflicts are recorded in .bzr. Two new commands 'conflicts'
+      and 'resolve' have needed added, which list and remove those 
+      merge conflicts respectively. A conflicted tree cannot be committed
+      in. Contributed by Aaron Bentley.
+
 
   INTERNALS:
 

=== modified file 'bzrlib/__init__.py'
--- bzrlib/__init__.py	2005-09-30 03:43:58 +0000
+++ bzrlib/__init__.py	2005-10-06 12:34:22 +0000
@@ -39,7 +39,7 @@
 del locale
 
 __copyright__ = "Copyright 2005 Canonical Development Ltd."
-__version__ = '0.10pre(newformat)'
+__version__ = '0.1rc1'
 
 
 def get_bzr_revision():

=== modified file 'bzrlib/branch.py'
--- bzrlib/branch.py	2005-10-06 06:31:33 +0000
+++ bzrlib/branch.py	2005-10-06 07:31:42 +0000
@@ -134,6 +134,7 @@
     def open(base):
         """Open an existing branch, rooted at 'base' (url)"""
         t = get_transport(base)
+        mutter("trying to open %r with transport %r", base, t)
         return _Branch(t)
 
     @staticmethod
@@ -465,7 +466,7 @@
             fmt = self.controlfile('branch-format', 'r').read()
         except NoSuchFile:
             raise NotBranchError(self.base)
-
+        mutter("got branch format %r", fmt)
         if fmt == BZR_BRANCH_FORMAT_5:
             self._branch_format = 5
         elif fmt == BZR_BRANCH_FORMAT_4:

=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2005-10-06 07:11:13 +0000
+++ bzrlib/builtins.py	2005-10-06 13:08:14 +0000
@@ -323,9 +323,10 @@
     If branches have diverged, you can use 'bzr merge' to pull the text changes
     from one into the other.
     """
+    takes_options = ['remember']
     takes_args = ['location?']
 
-    def run(self, location=None):
+    def run(self, location=None, remember=False):
         from bzrlib.merge import merge
         import tempfile
         from shutil import rmtree
@@ -337,7 +338,7 @@
             if stored_loc is None:
                 raise BzrCommandError("No pull location known or specified.")
             else:
-                print "Using last location: %s" % stored_loc
+                print "Using saved location: %s" % stored_loc
                 location = stored_loc
         cache_root = tempfile.mkdtemp()
         from bzrlib.errors import DivergedBranches
@@ -357,7 +358,7 @@
                     "  Try merge.")
                 
             merge(('.', -1), ('.', old_revno), check_clean=False)
-            if location != stored_loc:
+            if stored_loc is None or remember:
                 br_to.set_parent(location)
         finally:
             rmtree(cache_root)
@@ -997,7 +998,7 @@
     
     def run(self, message=None, file=None, verbose=True, selected_list=None,
             unchanged=False):
-        from bzrlib.errors import PointlessCommit
+        from bzrlib.errors import PointlessCommit, ConflictsInTree
         from bzrlib.msgeditor import edit_commit_message
         from bzrlib.status import show_status
         from cStringIO import StringIO
@@ -1005,6 +1006,7 @@
         b = Branch.open_containing('.')
         if selected_list:
             selected_list = [b.relpath(s) for s in selected_list]
+
             
         if message is None and not file:
             catcher = StringIO()
@@ -1034,6 +1036,9 @@
             # perhaps prepare the commit; get the message; then actually commit
             raise BzrCommandError("no changes to commit",
                                   ["use --unchanged to commit anyhow"])
+        except ConflictsInTree:
+            raise BzrCommandError("Conflicts detected in working tree.  "
+                'Use "bzr conflicts" to list, "bzr resolve FILE" to resolve.')
 
 
 class cmd_check(Command):
@@ -1234,13 +1239,18 @@
     takes_args = ['branch?']
     takes_options = ['revision', 'force', 'merge-type']
 
-    def run(self, branch='.', revision=None, force=False, 
+    def run(self, branch=None, revision=None, force=False, 
             merge_type=None):
         from bzrlib.merge import merge
         from bzrlib.merge_core import ApplyMerge3
         if merge_type is None:
             merge_type = ApplyMerge3
-
+        if branch is None:
+            branch = Branch.open_containing('.').get_parent()
+            if branch is None:
+                raise BzrCommandError("No merge location known or specified.")
+            else:
+                print "Using saved location: %s" % branch 
         if revision is None or len(revision) < 1:
             base = [None, None]
             other = [branch, -1]

=== modified file 'bzrlib/changeset.py'
--- bzrlib/changeset.py	2005-10-03 01:42:16 +0000
+++ bzrlib/changeset.py	2005-10-03 19:50:36 +0000
@@ -358,9 +358,10 @@
         else:
             assert(status == 1)
             def get_lines(filename):
-                my_file = file(base, "rb")
+                my_file = file(filename, "rb")
                 lines = my_file.readlines()
                 my_file.close()
+                return lines
             base_lines = get_lines(base)
             other_lines = get_lines(other)
             conflict_handler.merge_conflict(new_file, filename, base_lines, 

=== modified file 'bzrlib/commands.py'
--- bzrlib/commands.py	2005-10-04 02:30:58 +0000
+++ bzrlib/commands.py	2005-10-06 13:08:14 +0000
@@ -355,6 +355,7 @@
     'root':                   str,
     'no-backup':              None,
     'pattern':                str,
+    'remember':               None,
     }
 
 SHORT_OPTIONS = {
@@ -624,14 +625,10 @@
 def run_bzr_catch_errors(argv):
     try:
         try:
-            try:
-                return run_bzr(argv)
-            finally:
-                # do this here inside the exception wrappers to catch EPIPE
-                sys.stdout.flush()
-        #wrap common errors as CommandErrors.
-        except (NotBranchError,), e:
-            raise BzrCommandError(str(e))
+            return run_bzr(argv)
+        finally:
+            # do this here inside the exception wrappers to catch EPIPE
+            sys.stdout.flush()
     except BzrCommandError, e:
         # command line syntax error, etc
         log_error(str(e))
@@ -653,6 +650,8 @@
             bzrlib.trace.note('broken pipe')
             return 2
         else:
+            ## import pdb
+            ## pdb.pm()
             bzrlib.trace.log_exception()
             return 2
 

=== modified file 'bzrlib/commit.py'
--- bzrlib/commit.py	2005-10-06 07:03:33 +0000
+++ bzrlib/commit.py	2005-10-06 13:08:14 +0000
@@ -74,6 +74,7 @@
 from bzrlib.branch import gen_file_id
 from bzrlib.errors import (BzrError, PointlessCommit,
                            HistoryMissing,
+                           ConflictsInTree
                            )
 from bzrlib.revision import Revision
 from bzrlib.trace import mutter, note, warning
@@ -97,26 +98,38 @@
 
 class NullCommitReporter(object):
     """I report on progress of a commit."""
-    def added(self, path):
-        pass
-
-    def removed(self, path):
-        pass
-
-    def renamed(self, old_path, new_path):
-        pass
-
+
+    def snapshot_change(self, change, path):
+        pass
+
+    def completed(self, revno, rev_id):
+        pass
+
+    def deleted(self, file_id):
+        pass
+
+    def escaped(self, escape_count, message):
+        pass
+
+    def missing(self, path):
+        pass
 
 class ReportCommitToLog(NullCommitReporter):
-    def added(self, path):
-        note('added %s', path)
-
-    def removed(self, path):
-        note('removed %s', path)
-
-    def renamed(self, old_path, new_path):
-        note('renamed %s => %s', old_path, new_path)
-
+
+    def snapshot_change(self, change, path):
+        note("%s %s", change, path)
+
+    def completed(self, revno, rev_id):
+        note('committed r%d {%s}', revno, rev_id)
+    
+    def deleted(self, file_id):
+        note('deleted %s', file_id)
+
+    def escaped(self, escape_count, message):
+        note("replaced %d control characters in message", escape_count)
+
+    def missing(self, path):
+        note('missing %s', path)
 
 class Commit(object):
     """Task of committing a new revision.
@@ -218,10 +231,12 @@
                     or self.new_inv != self.basis_inv):
                 raise PointlessCommit()
 
+            if len(list(self.work_tree.iter_conflicts()))>0:
+                raise ConflictsInTree
+
             self._record_inventory()
             self._make_revision()
-            note('committed r%d {%s}', (self.branch.revno() + 1),
-                 self.rev_id)
+            self.reporter.completed(self.branch.revno()+1, self.rev_id)
             self.branch.append_revision(self.rev_id)
             self.branch.set_pending_merges([])
         finally:
@@ -252,7 +267,7 @@
             lambda match: match.group(0).encode('unicode_escape'),
             self.message)
         if escape_count:
-            note("replaced %d control characters in message", escape_count)
+            self.reporter.escaped(escape_count, self.message)
 
     def _gather_parents(self):
         """Record the parents of a merge for merge detection."""
@@ -274,7 +289,7 @@
             mutter('commit parent revision {%s}', parent_id)
             if not self.branch.has_revision(parent_id):
                 if parent_id == self.branch.last_revision():
-                    warning("parent is pissing %r", parent_id)
+                    warning("parent is missing %r", parent_id)
                     raise HistoryMissing(self.branch, 'revision', parent_id)
                 else:
                     mutter("commit will ghost revision %r", parent_id)
@@ -311,12 +326,12 @@
             if specific and not is_inside_any(specific, path):
                 continue
             if not self.work_tree.has_filename(path):
-                note('missing %s', path)
-                deleted_ids.append(ie.file_id)
+                self.reporter.missing(path)
+                deleted_ids.append((path, ie.file_id))
         if deleted_ids:
-            for file_id in deleted_ids:
-                if file_id in self.work_inv:
-                    del self.work_inv[file_id]
+            deleted_ids.sort(reverse=True)
+            for path, file_id in deleted_ids:
+                del self.work_inv[file_id]
             self.branch._write_inventory(self.work_inv)
 
     def _store_snapshot(self):
@@ -338,7 +353,7 @@
                                      self.work_tree, self.weave_store)
             else:
                 change = "unchanged"
-            note("%s %s", change, path)
+            self.reporter.snapshot_change(change, path)
 
     def _populate_new_inv(self):
         """Build revision inventory.
@@ -393,7 +408,7 @@
     def _report_deletes(self):
         for file_id in self.basis_inv:
             if file_id not in self.new_inv:
-                note('deleted %s', self.basis_inv.id2path(file_id))
+                self.reporter.deleted(self.basis_inv.id2path(file_id))
 
 def _gen_revision_id(branch, when):
     """Return new revision-id."""

=== modified file 'bzrlib/errors.py'
--- bzrlib/errors.py	2005-10-05 10:02:53 +0000
+++ bzrlib/errors.py	2005-10-06 13:08:14 +0000
@@ -23,16 +23,21 @@
 # exceptions 
 class BzrError(StandardError):
     def __str__(self):
+        # XXX: Should we show the exception class in 
+        # exceptions that don't provide their own message?  
+        # maybe it should be done at a higher level
+        ## n = self.__class__.__name__ + ': '
+        n = ''
         if len(self.args) == 1:
-            return self.args[0]
+            return n + self.args[0]
         elif len(self.args) == 2:
             # further explanation or suggestions
             try:
-                return '\n  '.join([self.args[0]] + self.args[1])
+                return n + '\n  '.join([self.args[0]] + self.args[1])
             except TypeError:
-                return "%r" % self
+                return n + "%r" % self
         else:
-            return `self.args`
+            return n + `self.args`
 
 
 class BzrCheckError(BzrError):
@@ -50,12 +55,14 @@
 
 class BzrCommandError(BzrError):
     # Error from malformed user command
-    pass
+    def __str__(self):
+        return self.args[0]
 
 
 class NotBranchError(BzrError):
     """Specified path is not in a branch"""
-    pass
+    def __str__(self):
+        return 'not a branch: %s' % self.args[0]
 
 
 class NotVersionedError(BzrError):
@@ -207,8 +214,10 @@
 class NoSuchFile(TransportError, IOError):
     """A get() was issued for a file that doesn't exist."""
 
+    # XXX: Is multiple inheritance for exceptions really needed?
+
     def __str__(self):
-        return self.msg
+        return 'no such file: ' + self.msg
 
     def __init__(self, msg=None, orig_error=None):
         import errno
@@ -221,6 +230,7 @@
 
     mkdir() can throw this, but put() just overwites existing files.
     """
+    # XXX: Is multiple inheritance for exceptions really needed?
     def __init__(self, msg=None, orig_error=None):
         import errno
         TransportError.__init__(self, msg=msg, orig_error=orig_error)
@@ -234,3 +244,6 @@
     """The connection has been closed."""
     pass
 
+class ConflictsInTree(BzrError):
+    def __init__(self):
+        BzrError.__init__(self, "Working tree has conflicts.")

=== added file 'bzrlib/plugins/conflicts.py'
--- bzrlib/plugins/conflicts.py	1970-01-01 00:00:00 +0000
+++ bzrlib/plugins/conflicts.py	2005-10-06 04:20:44 +0000
@@ -0,0 +1,68 @@
+# Copyright (C) 2005 by Aaron Bentley
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+import bzrlib.status
+from bzrlib.branch import Branch
+from bzrlib.errors import BzrCommandError
+from bzrlib.commands import register_command
+from bzrlib.workingtree import CONFLICT_SUFFIXES
+import os
+import errno
+
+class cmd_conflicts(bzrlib.commands.Command):
+    """List files with conflicts.
+    (conflicts are determined by the presence of .BASE .TREE, and .OTHER 
+    files.)
+    """
+    def run(self):
+        for path in Branch.open_containing('.').working_tree().iter_conflicts():
+            print path
+
+register_command(cmd_conflicts)
+
+class cmd_resolve(bzrlib.commands.Command):
+    """Mark a conflict as resolved.
+    """
+    takes_args = ['file*']
+    takes_options = ['all']
+    def run(self, file_list=None, all=False):
+        if file_list is None:
+            if not all:
+                raise BzrCommandError(
+                    "command 'resolve' needs one or more FILE, or --all")
+            tree = Branch.open_containing('.').working_tree()
+            file_list = list(tree.abspath(f) for f in tree.iter_conflicts())
+        else:
+            if all:
+                raise BzrCommandError(
+                    "If --all is specified, no FILE may be provided")
+        for filename in file_list:
+            failures = 0
+            for suffix in CONFLICT_SUFFIXES:
+                try:
+                    os.unlink(filename+suffix)
+                except OSError, e:
+                    if e.errno != errno.ENOENT:
+                        raise
+                    else:
+                        failures += 1
+            if failures == len(CONFLICT_SUFFIXES):
+                if not os.path.exists(filename):
+                    print "%s does not exist" % filename
+                else:
+                    print "%s is not conflicted" % filename
+                        
+register_command(cmd_resolve)

=== modified file 'bzrlib/selftest/__init__.py'
--- bzrlib/selftest/__init__.py	2005-10-06 11:26:51 +0000
+++ bzrlib/selftest/__init__.py	2005-10-06 22:15:52 +0000
@@ -488,6 +488,7 @@
                    'bzrlib.selftest.testgraph',
                    'bzrlib.selftest.testworkingtree',
                    'bzrlib.selftest.test_upgrade',
+                   'bzrlib.selftest.test_conflicts',
                    ]
 
     for m in (bzrlib.store, bzrlib.inventory, bzrlib.branch,

=== modified file 'bzrlib/selftest/blackbox.py'
--- bzrlib/selftest/blackbox.py	2005-10-06 05:13:21 +0000
+++ bzrlib/selftest/blackbox.py	2005-10-06 13:08:14 +0000
@@ -308,7 +308,41 @@
         a.get_revision_xml(b.last_revision())
         self.log('pending merges: %s', a.pending_merges())
         #        assert a.pending_merges() == [b.last_revision()], "Assertion %s %s" \
-        #        % (a.pending_merges(), b.last_revision())
+        #        % (a.pending_merges(), b.last_patch())
+
+    def test_merge_with_missing_file(self):
+        """Merge handles missing file conflicts"""
+        os.mkdir('a')
+        os.chdir('a')
+        os.mkdir('sub')
+        print >> file('sub/a.txt', 'wb'), "hello"
+        print >> file('b.txt', 'wb'), "hello"
+        print >> file('sub/c.txt', 'wb'), "hello"
+        self.runbzr('init')
+        self.runbzr('add')
+        self.runbzr(('commit', '-m', 'added a'))
+        self.runbzr('branch . ../b')
+        print >> file('sub/a.txt', 'ab'), "there"
+        print >> file('b.txt', 'ab'), "there"
+        print >> file('sub/c.txt', 'ab'), "there"
+        self.runbzr(('commit', '-m', 'Added there'))
+        os.unlink('sub/a.txt')
+        os.unlink('sub/c.txt')
+        os.rmdir('sub')
+        os.unlink('b.txt')
+        self.runbzr(('commit', '-m', 'Removed a.txt'))
+        os.chdir('../b')
+        print >> file('sub/a.txt', 'ab'), "something"
+        print >> file('b.txt', 'ab'), "something"
+        print >> file('sub/c.txt', 'ab'), "something"
+        self.runbzr(('commit', '-m', 'Modified a.txt'))
+        self.runbzr('merge ../a/')
+        assert os.path.exists('sub/a.txt.THIS')
+        assert os.path.exists('sub/a.txt.BASE')
+        os.chdir('../a')
+        self.runbzr('merge ../b/')
+        assert os.path.exists('sub/a.txt.OTHER')
+        assert os.path.exists('sub/a.txt.BASE')
 
     def test_merge_with_missing_file(self):
         """Merge handles missing file conflicts"""
@@ -393,6 +427,34 @@
         self.runbzr('commit -m blah8 --unchanged')
         self.runbzr('pull ../b')
         self.runbzr('pull ../b')
+
+    def test_locations(self):
+        """Using and remembering different locations"""
+        os.mkdir('a')
+        os.chdir('a')
+        self.runbzr('init')
+        self.runbzr('commit -m unchanged --unchanged')
+        self.runbzr('pull', retcode=1)
+        self.runbzr('merge', retcode=1)
+        self.runbzr('branch . ../b')
+        os.chdir('../b')
+        self.runbzr('pull')
+        self.runbzr('branch . ../c')
+        self.runbzr('pull ../c')
+        self.runbzr('merge')
+        os.chdir('../a')
+        self.runbzr('pull ../b')
+        self.runbzr('pull')
+        self.runbzr('pull ../c')
+        self.runbzr('branch ../c ../d')
+        shutil.rmtree('../c')
+        self.runbzr('pull')
+        os.chdir('../b')
+        self.runbzr('pull')
+        os.chdir('../d')
+        self.runbzr('pull', retcode=1)
+        self.runbzr('pull ../a --remember')
+        self.runbzr('pull')
         
     def test_add_reports(self):
         """add command prints the names of added files."""
@@ -413,6 +475,40 @@
         self.assertEquals(out, '')
         err.index('unknown command')
 
+    def test_conflicts(self):
+        """Handling of merge conflicts"""
+        os.mkdir('base')
+        os.chdir('base')
+        file('hello', 'wb').write("hi world")
+        file('answer', 'wb').write("42")
+        self.runbzr('init')
+        self.runbzr('add')
+        self.runbzr('commit -m base')
+        self.runbzr('branch . ../other')
+        self.runbzr('branch . ../this')
+        os.chdir('../other')
+        file('hello', 'wb').write("Hello.")
+        file('answer', 'wb').write("Is anyone there?")
+        self.runbzr('commit -m other')
+        os.chdir('../this')
+        file('hello', 'wb').write("Hello, world")
+        self.runbzr('mv answer question')
+        file('question', 'wb').write("What do you get when you multiply six"
+                                   "times nine?")
+        self.runbzr('commit -m this')
+        self.runbzr('merge ../other')
+        result = self.runbzr('conflicts', backtick=1)
+        self.assertEquals(result, "hello\nquestion\n")
+        result = self.runbzr('status', backtick=1)
+        assert "conflicts:\n  hello\n  question\n" in result, result
+        self.runbzr('resolve hello')
+        result = self.runbzr('conflicts', backtick=1)
+        self.assertEquals(result, "question\n")
+        self.runbzr('commit -m conflicts', retcode=1)
+        self.runbzr('resolve --all')
+        result = self.runbzr('conflicts', backtick=1)
+        self.runbzr('commit -m conflicts')
+        self.assertEquals(result, "")
 
 def listdir_sorted(dir):
     L = os.listdir(dir)

=== added file 'bzrlib/selftest/test_conflicts.py'
--- bzrlib/selftest/test_conflicts.py	1970-01-01 00:00:00 +0000
+++ bzrlib/selftest/test_conflicts.py	2005-10-06 03:15:08 +0000
@@ -0,0 +1,44 @@
+# Copyright (C) 2005 by Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+import os
+
+from bzrlib.selftest import TestCaseInTempDir
+from bzrlib.branch import Branch
+from bzrlib.commit import Commit
+
+
+# TODO: Test commit with some added, and added-but-missing files
+
+class TestConflicts(TestCaseInTempDir):
+
+    def test_conflicts(self):
+        """Conflicts are detected properly"""
+        b = Branch.initialize('.')
+        file('hello', 'w').write('hello world')
+        file('hello.BASE', 'w').write('hello world')
+        file('hello.sploo.BASE', 'w').write('yellow world')
+        tree = b.working_tree()
+        self.assertEqual(len(list(tree.list_files())), 3)
+        conflicts = list(tree.iter_conflicts())
+        self.assertEqual(len(conflicts), 2)
+        assert 'hello' in conflicts
+        assert 'hello.sploo' in conflicts
+        os.unlink('hello.BASE')
+        os.unlink('hello.sploo.BASE')
+        self.assertEqual(len(list(tree.iter_conflicts())), 0)
+

=== modified file 'bzrlib/selftest/test_weave.py'
--- bzrlib/selftest/test_weave.py	2005-10-05 04:38:09 +0000
+++ bzrlib/selftest/test_weave.py	2005-10-06 10:53:12 +0000
@@ -840,6 +840,58 @@
         eq(wa.get_lines('b1'),
            ['hello\n', 'pale blue\n', 'world\n'])
 
+    def test_join_parent_disagreement(self):
+        """Cannot join weaves with different parents for a version."""
+        wa = Weave()
+        wb = Weave()
+        wa.add('v1', [], ['hello\n'])
+        wb.add('v0', [], [])
+        wb.add('v1', ['v0'], ['hello\n'])
+        self.assertRaises(WeaveError,
+                          wa.join, wb)
+
+    def test_join_text_disagreement(self):
+        """Cannot join weaves with different texts for a version."""
+        wa = Weave()
+        wb = Weave()
+        wa.add('v1', [], ['hello\n'])
+        wb.add('v1', [], ['not\n', 'hello\n'])
+        self.assertRaises(WeaveError,
+                          wa.join, wb)
+
+    def test_join_unordered(self):
+        """Join weaves where indexes differ.
+        
+        The source weave contains a different version at index 0."""
+        wa = self.weave1.copy()
+        wb = Weave()
+        wb.add('x1', [], ['line from x1\n'])
+        wb.add('v1', [], ['hello\n'])
+        wb.add('v2', ['v1'], ['hello\n', 'world\n'])
+        wa.join(wb)
+        eq = self.assertEquals
+        eq(sorted(wa.iter_names()), ['v1', 'v2', 'v3', 'x1',])
+        eq(wa.get_text('x1'), 'line from x1\n')
+
+    def test_join_with_ghosts(self):
+        """Join that inserts parents of an existing revision.
+
+        This can happen when merging from another branch who
+        knows about revisions the destination does not.  In 
+        this test the second weave knows of an additional parent of 
+        v2.  Any revisions which are in common still have to have the 
+        same text."""
+        return ###############################
+        wa = self.weave1.copy()
+        wb = Weave()
+        wb.add('x1', [], ['line from x1\n'])
+        wb.add('v1', [], ['hello\n'])
+        wb.add('v2', ['v1', 'x1'], ['hello\n', 'world\n'])
+        wa.join(wb)
+        eq = self.assertEquals
+        eq(sorted(wa.iter_names()), ['v1', 'v2', 'v3', 'x1',])
+        eq(wa.get_text('x1'), 'line from x1\n')
+
 
 if __name__ == '__main__':
     import sys

=== modified file 'bzrlib/status.py'
--- bzrlib/status.py	2005-10-03 01:53:47 +0000
+++ bzrlib/status.py	2005-10-06 13:08:14 +0000
@@ -1,19 +1,22 @@
 # (C) 2005 Canonical Ltd
-
+#
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
-
+#
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
-
+#
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
+import sys
+from bzrlib.osutils import is_inside_any
+from bzrlib.delta import compare_trees
 
 
 def show_status(branch, show_unchanged=False,
@@ -44,10 +47,6 @@
         If one revision show compared it with working tree.
         If two revisions show status between first and second.
     """
-    import sys
-    from bzrlib.osutils import is_inside_any
-    from bzrlib.delta import compare_trees
-
     if to_file == None:
         to_file = sys.stdout
     
@@ -82,15 +81,10 @@
                    show_unchanged=show_unchanged)
 
         if new_is_working_tree:
+            conflicts = new.iter_conflicts()
             unknowns = new.unknowns()
-            done_header = False
-            for path in unknowns:
-                if specific_files and not is_inside_any(specific_files, path):
-                    continue
-                if not done_header:
-                    print >>to_file, 'unknown:'
-                    done_header = True
-                print >>to_file, ' ', path
+            list_paths('unknown', unknowns, specific_files, to_file)
+            list_paths('conflicts', conflicts, specific_files, to_file)
             if show_pending and len(branch.pending_merges()) > 0:
                 print >>to_file, 'pending merges:'
                 for merge in branch.pending_merges():
@@ -98,3 +92,12 @@
     finally:
         branch.unlock()
         
+def list_paths(header, paths, specific_files, to_file):
+    done_header = False
+    for path in paths:
+        if specific_files and not is_inside_any(specific_files, path):
+            continue
+        if not done_header:
+            print >>to_file, '%s:' % header
+            done_header = True
+        print >>to_file, ' ', path

=== modified file 'bzrlib/trace.py'
--- bzrlib/trace.py	2005-10-06 04:03:50 +0000
+++ bzrlib/trace.py	2005-10-06 07:28:41 +0000
@@ -164,10 +164,9 @@
     if msg == None:
         ei = sys.exc_info()
         msg = str(ei[1])
-
     if msg and (msg[-1] == '\n'):
         msg = msg[:-1]
-        
+    ## msg = "(%s) %s" % (str(type(ei[1])), msg)
     _bzr_logger.exception(msg)
 
 

=== modified file 'bzrlib/weave.py'
--- bzrlib/weave.py	2005-10-05 04:38:09 +0000
+++ bzrlib/weave.py	2005-10-06 08:11:46 +0000
@@ -805,6 +805,12 @@
 
     def _check_version_consistent(self, other, other_idx, name):
         """Check if a version in consistent in this and other.
+
+        To be consistent it must have:
+
+         * the same text
+         * the same direct parents (by name, not index, and disregarding
+           order)
         
         If present & correct return True;
         if not present in self return False; 
@@ -812,11 +818,19 @@
         this_idx = self._name_map.get(name, -1)
         if this_idx != -1:
             if self._sha1s[this_idx] != other._sha1s[other_idx]:
-                raise WeaveError("inconsistent texts for version {%s} in %r and %r"
-                                 % (name, self, other))
-            elif set(self._parents[this_idx]) != set(other._parents[other_idx]):
-                raise WeaveError("inconsistent parents for version {%s} in %r and %r"
-                                 % (name, self, other))
+                raise WeaveError("inconsistent texts for version {%s} "
+                                 "when joining weaves"
+                                 % (name))
+            self_parents = self._parents[this_idx]
+            other_parents = other._parents[other_idx]
+            n1 = [self._names[i] for i in self_parents]
+            n2 = [other._names[i] for i in other_parents]
+            n1.sort()
+            n2.sort()
+            if n1 != n2:
+                raise WeaveError("inconsistent parents for version {%s}: "
+                                 "%s vs %s"
+                                 % (name, n1, n2))
             else:
                 return True         # ok!
         else:

=== modified file 'bzrlib/workingtree.py'
--- bzrlib/workingtree.py	2005-10-03 03:30:02 +0000
+++ bzrlib/workingtree.py	2005-10-06 04:29:10 +0000
@@ -275,6 +275,15 @@
             if not self.is_ignored(subp):
                 yield subp
 
+    def iter_conflicts(self):
+        conflicted = set()
+        for path in (s[0] for s in self.list_files()):
+            stem = get_conflicted_stem(path)
+            if stem is None:
+                continue
+            if stem not in conflicted:
+                conflicted.add(stem)
+                yield stem
 
     def extras(self):
         """Yield all unknown files in this WorkingTree.
@@ -366,4 +375,9 @@
                     return pat
         else:
             return None
-        
+
+CONFLICT_SUFFIXES = ('.THIS', '.BASE', '.OTHER')
+def get_conflicted_stem(path):
+    for suffix in CONFLICT_SUFFIXES:
+        if path.endswith(suffix):
+            return path[:-len(suffix)]

=== modified file 'setup.py'
--- setup.py	2005-10-06 05:13:21 +0000
+++ setup.py	2005-10-06 12:34:22 +0000
@@ -51,7 +51,7 @@
 ########################
 
 setup(name='bzr',
-      version='0.0.6',
+      version='0.1',
       author='Martin Pool',
       author_email='mbp@sourcefrog.net',
       url='http://www.bazaar-ng.org/',

